package cn.sh.stc.sict.theme.config;

import cn.sh.stc.sict.theme.common.crypto.EncryptionRequest;
import cn.sh.stc.sict.theme.common.crypto.EncryptionUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdvice;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;

/**
 * @author gao
 * @date 2023/05/30 13:55
 */
@Slf4j
@ControllerAdvice
@RequiredArgsConstructor
public class DecodeRequestAdvice implements RequestBodyAdvice {
    private final ObjectMapper objectMapper;

    @Override
    public boolean supports(MethodParameter methodParameter, Type type, Class<? extends HttpMessageConverter<?>> aClass) {
        // 判断方法是否需要进行解密
        return true;
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage httpInputMessage, MethodParameter methodParameter, Type type, Class<? extends HttpMessageConverter<?>> aClass) throws IOException {
        // 读取body对象,获取EncryptionRequest对象
        InputStream body = httpInputMessage.getBody();
        EncryptionRequest encryptionRequest = objectMapper.readValue(body, EncryptionRequest.class);
        if (encryptionRequest.getEncryptionKey() == null) {
            throw new IllegalArgumentException("encryption error,request is null");
        }
        // 转换成解密后的HttpInputMessage对象
        return new HttpInputMessage() {
            @Override
            public HttpHeaders getHeaders() {
                return httpInputMessage.getHeaders();
            }

            // 解密后的输入流
            @Override
            public InputStream getBody() {
                return EncryptionUtil.getDecodeInputStream(encryptionRequest);
            }
        };
    }

    @Override
    public Object afterBodyRead(Object o, HttpInputMessage httpInputMessage, MethodParameter methodParameter, Type type, Class<? extends HttpMessageConverter<?>> aClass) {
        return o;
    }

    @Override
    public Object handleEmptyBody(Object o, HttpInputMessage httpInputMessage, MethodParameter methodParameter, Type type, Class<? extends HttpMessageConverter<?>> aClass) {
        return o;
    }


}
